//
//  TWLoactionManager.m
//  TrailWalker
//
//  Created by Roger Riera Diaz on 20/04/13.
//  Copyright (c) 2013 Roger Riera Diaz. All rights reserved.
//

#import "TWLocationManager.h"
//
@interface TWLocationManager ()

@end


@implementation TWLocationManager

@synthesize counterTask;
@synthesize locationManager;

// Mètode per començar la captura de la posició
- (BOOL)startTracking {
    
    // Si el servei està deshabilitat retornem fals
    if ((![CLLocationManager locationServicesEnabled])
        || ([CLLocationManager authorizationStatus] == kCLAuthorizationStatusRestricted)
        || ([CLLocationManager authorizationStatus] == kCLAuthorizationStatusDenied))
    {
        NSLog(@"Location Service inactive");
        return FALSE;
    }
    
    
    MyManager *sharedManager = [MyManager sharedManager];
    
    // Realitzem una primera captura.
    [self startStandardLocation]; 
    
    if ([sharedManager.mySettingData.locService isEqual: @"Significant"]) {
        // Per mètode significant
        [self startSignificantChangeUpdates];
    }
    else {
        // Per mètode standard + timer
        int time = sharedManager.mySettingData.frequency.intValue ;
        
        counterTask = [[UIApplication sharedApplication]
                       beginBackgroundTaskWithExpirationHandler:^{
                       }];
        
        // Invalidem el temporitzador anterior en cas d'existir.
        [self.repeatingTimer invalidate];
        NSTimer *timer = [NSTimer scheduledTimerWithTimeInterval:time * 60
                                                          target:self selector:@selector(targetMethod:)
                                                        userInfo:[self userInfo]
                                                         repeats:YES];
        self.repeatingTimer = timer;
        
    }
    
    return TRUE;
    
}

// Parem les captures de posició
- (void)stopTracking {
    
    [self.repeatingTimer invalidate];
    self.repeatingTimer = nil;
    [locationManager stopUpdatingLocation];
    [locationManager stopMonitoringSignificantLocationChanges];
    
    TWSendJson *sendJson = [TWSendJson alloc];
    [sendJson processCaptures];
    
}

// Activem el mètode de canvi significatiu
- (void)startSignificantChangeUpdates{
    
    // Creem el location manager si encara no ha estat creat.
    if (nil == locationManager)
        NSLog(@"Instancia l'objecte locationManager");
    locationManager = [[CLLocationManager alloc] init];
    
    locationManager.delegate = self;
    
    // Aquests 2 mètodes no són utilitzats pel significant change update
    [locationManager setDistanceFilter:kCLDistanceFilterNone];
    [locationManager setDesiredAccuracy:kCLLocationAccuracyBest];
    [locationManager startMonitoringSignificantLocationChanges];
    
}

// Activem el mètode de localització estàndar
- (void)startStandardLocation {
    
    // Creem el location manager si encara no ha estat creat.
    if (nil == locationManager)
        NSLog(@"Instancia l'objecte locationManager");
    locationManager = [[CLLocationManager alloc] init];
    
    locationManager.delegate = self;
    MyManager *sharedManager = [MyManager sharedManager];
    
    // Aquests 2 mètodes només són utilitzats per l'standard location
    [locationManager setDistanceFilter:sharedManager.mySettingData.accuracy.intValue];//distanceFilter];
    [locationManager setDesiredAccuracy:kCLLocationAccuracyBest];
    [locationManager startUpdatingLocation];
    
}

// Mètodes executats pel timer
- (NSDictionary *)userInfo {
    return @{ @"StartDate" : [NSDate date] };
}

- (void)targetMethod:(NSTimer*)theTimer {
    NSDate *startDate = [[theTimer userInfo] objectForKey:@"StartDate"];
    NSLog(@"Loop: ini %@", startDate);
    
    [self startStandardLocation];
}

#pragma mark CLLocationManagerDelegate

// Mètode de la classe delegada per a l'actualització d'una ubicació
- (void)locationManager:(CLLocationManager *)manager
     didUpdateLocations:(NSArray *)locations {
    
    MyManager *sharedManager = [MyManager sharedManager];
    CLLocation *location = [locations lastObject];
    
    // Per l'estandar parem la captura fins la següent iteració del timer
    [locationManager stopUpdatingLocation];
    
    TWCaptura *capture = [[TWCaptura alloc] init];
    [capture createCapture:location];
    
    // Afegim la captura a la llista complerta
    [sharedManager setMyLastCapture:capture];
    [sharedManager.myCaptureList addCapture];
    
    // Avisem el Centre de Notificacions per actualitzar la pantalla de captures
    [[NSNotificationCenter defaultCenter] postNotificationName:@"dealNotification" object: nil];
    
    // Cridem a la classe sendJson per que enviï totes les captures pendents, no només la actual
    TWSendJson *sendJson = [TWSendJson alloc];
    [sendJson processCaptures];
}

// Si l'usuari denega la autorització de captura d'ubicació, enviem Notificació per fer apareixer un avís i apagar el procés
-(void)locationManager:(CLLocationManager *)manager
    didChangeAuthorizationStatus:(CLAuthorizationStatus)status {
    
    if (status == kCLAuthorizationStatusDenied) {
        [[NSNotificationCenter defaultCenter] postNotificationName:@"dealDeniedNotification" object: nil];
    }
    
}


@end
