//
//  TWFirstViewController.m
//  TrailWalker
//
//  Created by Roger Riera Diaz on 17/04/13.
//  Copyright (c) 2013 Roger Riera Diaz. All rights reserved.
//

#import "TWMapViewController.h"

#define kAlertViewOne 1
#define kAlertViewTwo 2

@interface TWMapViewController ()

@end

@implementation TWMapViewController
@synthesize mylocationManager;

UISwitch *mySwitch;


- (void)viewDidLoad
{
    [super viewDidLoad];

    // Carreguem el mapa base a partir del MapService d'ESRI
	self.mapView.layerDelegate = self;
	AGSTiledMapServiceLayer *tiledLayer = [[AGSTiledMapServiceLayer alloc] initWithURL:[NSURL URLWithString:kTiledMapServiceURL]];
	[self.mapView addMapLayer:tiledLayer withName:@"Tiled Layer"];
    
    
    // Definim des del codi, el Switch ja que no és un element natiu de la barra de navegació
    mySwitch = [[UISwitch alloc] init];
    [mySwitch addTarget:self action:@selector(flipSwitch:) forControlEvents:UIControlEventValueChanged];
    self.navigationItem.rightBarButtonItem = [[UIBarButtonItem alloc] initWithCustomView: mySwitch];
    
    mylocationManager = [[TWLocationManager alloc] init];
    // Habilitem el Centre de Notificacions per rebre avís en cas que hagin deshabilitat el sistema de localització de l'iPhone
    [[NSNotificationCenter defaultCenter] addObserver: self selector: @selector(disableLocation) name:@"dealDeniedNotification" object: nil];

}

-(void)viewWillAppear:(BOOL)animated {
    
    [super viewWillAppear:animated];
    
    // La primera vegada que arrenca, obliguem la introducció del nom de l'equip
    MyManager *sharedManager = [MyManager sharedManager];
    if ([sharedManager.mySettingData.team length] == 0)
    {
        [self showInputTeam];
    }
    
}

- (void)showInputTeam
{
    // Aquesta és el dialeg per la introducció inicial del nom de l'equip
    UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"teamTextFieldTitle", @"")
                                                        message:NSLocalizedString(@"teamTextEnterMessage", @"")
                                                       delegate:self
                                              cancelButtonTitle:nil
                                              otherButtonTitles:@"Ok", nil];
    
    alertView.alertViewStyle = UIAlertViewStylePlainTextInput;
    alertView.tag = kAlertViewOne;
    [alertView show];
}

// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown;
}


- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
    self.mapView = nil;
}


- (void)dealloc {
	
}

// El métode que s'executa cada cop que es prem el switch de captura
- (IBAction)flipSwitch:sender {
    UISwitch *onoff = (UISwitch *) sender;
    NSLog(@"%@", onoff.on ? @"Switch On" : @"Switch Off");
        
    if (onoff.on) {
        // Si no està actiu no permetem arrancar el seguiment
        if (![mylocationManager startTracking]) {
            [self disableLocation];
        }
    } else {
        // Si apaguem parem les captures
        [mylocationManager stopTracking];
    }

}

// Accions a realitzar quan el sistema de localització apagat
-(void)disableLocation {
    [mylocationManager stopTracking];
    [mySwitch setOn:FALSE];
    [self showAlertLocationDisabled];
}

// Alerta de sistema de localització apagat
-(void) showAlertLocationDisabled {
    UIAlertView *alertView = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"errorAlertTitle", @"")
                                                        message:NSLocalizedString(@"detailDisabledMessage", @"")
                                                       delegate:self
                                              cancelButtonTitle:nil
                                              otherButtonTitles:@"Ok", nil];
    
    alertView.alertViewStyle = UIAlertViewStyleDefault;
    // Com que utilitzem 2 alertes en aquest classe identifiquem quina és assignant tag
    alertView.tag = kAlertViewTwo;
    [alertView show];
}

#pragma mark UIAlertViewDelegate methods

// Si és l'alerta per la entrada de l'equip, validem que el text sigui entre 1 i 50
- (BOOL)alertViewShouldEnableFirstOtherButton:(UIAlertView *)alertView
{
    if(alertView.tag == kAlertViewOne)
    {
        NSString *inputText = [[alertView textFieldAtIndex:0] text];
        if( ([inputText length] <= 0) || ([inputText length] > 50))
        {
            return NO;
        }
    }
    return YES;
}

// Si apretem el botó OK registrem el nom de l'equip a través de la Singleton
-(void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex {
    if(alertView.tag == kAlertViewOne) 
    {
        UITextField *teamTextField = [alertView textFieldAtIndex:0];
        NSLog(@"Team Name %@", teamTextField.text);
        MyManager *sharedManager = [MyManager sharedManager];
        [sharedManager.mySettingData setTeam:teamTextField.text];
        [sharedManager.mySettingData setDataOnDict];
    }
}

#pragma mark AGSMapViewLayerDelegate methods

// Metode delegat quan el mapa aconsegueix carregar el servei ESRI
-(void) mapViewDidLoad:(AGSMapView*)mapView {
    
	// comment to disable the GPS on start up
	[self.mapView.locationDisplay startDataSource];
    self.mapView.locationDisplay.autoPanMode = AGSLocationDisplayAutoPanModeDefault;
}


@end
