//
//  TWSendJson.m
//  TrailWalker
//
//  Created by Roger Riera Diaz on 02/05/13.
//  Copyright (c) 2013 Roger Riera Diaz. All rights reserved.
//

#import "TWSendJson.h"


@implementation TWSendJson

@synthesize receivedData = _receivedData;

// Mètode per enviar totes les captures no ateses
-(void) processCaptures {
    MyManager *sharedManager = [MyManager sharedManager];
    NSMutableArray *unnattendedList = nil;
    
    unnattendedList = [sharedManager.myCaptureList getUnnattendedList];
    
    if ((unnattendedList != nil) && ([unnattendedList count] != 0)) {
        TWCaptura *capt = [unnattendedList objectAtIndex:0];
        // Cridem el mètode sendJsonCapture per a cada capture de la llista unnattendedList
        [self sendJsonCapture:capt];
    }
}

// Mètode per enviar la captura d'ubicació al servei REST d'ArcGIS
-(void) sendJsonCapture:(TWCaptura *)capture {
    
    // Transformem la capture en un diccionari
    NSDictionary *jsonDict = [self getNSDictFromCapture:capture];
    
    // Convertim a JSon i codifiquem segons x-www-form-urlencoded
    NSString *jsonRequest = [self convertNSDictToEncodedJson:jsonDict];
    
    // La url del servei Add Feature
    NSURL *url = [NSURL URLWithString:@"http://napols.uoc.es/arcgis/rest/services/TrailWalker/FeatureServer/0/addFeatures"];
    
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:url
                                                           cachePolicy:NSURLRequestUseProtocolCachePolicy timeoutInterval:60.0];
    
    NSData *requestData = [NSData dataWithBytes:[jsonRequest UTF8String] length:[jsonRequest length]];
    
    // Establim els paràmetres de connexió
    [request setHTTPMethod:@"POST"];
    [request setValue:@"application/json" forHTTPHeaderField:@"Accept"];
    [request setValue:@"application/x-www-form-urlencoded" forHTTPHeaderField:@"Content-Type"];
    [request setValue:[NSString stringWithFormat:@"%d", [requestData length]] forHTTPHeaderField:@"Content-Length"];
    [request setHTTPBody: requestData];
    
    // I realitzem la crida HTTP POST amb la informació JSON continguda
    NSURLConnection *connection = [[NSURLConnection alloc]initWithRequest:request delegate:self];
    if (connection) {
        receivedData = [NSMutableData data];
    }

}

-(NSDictionary *) getNSDictFromCapture:(TWCaptura *)capture {
    
    NSDictionary *jsonDict = nil;
    
    NSDateFormatter *dateFormat = [[NSDateFormatter alloc] init];
    [dateFormat setDateFormat:@"yyyy-MM-dd HH:mm:ss"];
    NSString *dateString = [dateFormat stringFromDate:capture.timeStampCapt];
    
    // En un sol diccionari, registrem tots els valors necessaris per crear el json
    NSMutableDictionary *dict = [NSMutableDictionary dictionary];
    
    [dict setObject:capture.longitudeCapt.stringValue forKey:@"longitude"];
    [dict setObject:capture.latitudeCapt.stringValue forKey:@"latitude"];
    [dict setObject:@"104910" forKey:@"wkid"];
    [dict setObject:capture.teamCodeCapt forKey:@"codiequip"];
    [dict setObject:capture.teamCapt forKey:@"equip"];
    [dict setObject:capture.batteryCapt.stringValue forKey:@"bateria"];
    [dict setObject:dateString forKey:@"entrada"];
    [dict setObject:@"iOS" forKey:@"tipus"];
    [dict setObject:[capture.uuidCodeCapt substringToIndex:36] forKey:@"uuid"];
    
    // Establim diccionari amb el sistema de referencia
    NSArray *objects = [NSArray arrayWithObjects:
                        [dict objectForKey:@"wkid"],
                        nil];
    
    NSArray *keys = [NSArray arrayWithObjects:
                     @"wkid",
                     nil];
    
    NSDictionary *wkidDict = [NSDictionary dictionaryWithObjects:objects forKeys:keys];
    
    // Establim diccionari amb la geometria
    objects = [NSArray arrayWithObjects:
                        [dict objectForKey:@"longitude"],
                        [dict objectForKey:@"latitude"],
                        wkidDict,
                        nil];
    
    keys = [NSArray arrayWithObjects:
                     @"x",
                     @"y",
                     @"spatialReference",
                     nil];
    
    NSDictionary *geomDict = [NSDictionary dictionaryWithObjects:objects forKeys:keys];
    
    // Establim diccionari amb els atributs
    objects = [NSArray arrayWithObjects:
                        [dict objectForKey:@"uuid"],
                        [dict objectForKey:@"tipus"],
                        [dict objectForKey:@"entrada"],
                        [dict objectForKey:@"bateria"],
                        [dict objectForKey:@"equip"],
                        [dict objectForKey:@"codiequip"],
                        nil];
    
    keys = [NSArray arrayWithObjects:
                     @"uuid",
                     @"tipus",
                     @"entrada",
                     @"bateria",
                     @"equip",
                     @"codiequip",
                     nil];
    NSDictionary *attributesDict = [NSDictionary dictionaryWithObjects:objects forKeys:keys];
    
    jsonDict = [NSDictionary dictionaryWithObjects:[NSArray arrayWithObjects:attributesDict, geomDict, nil] forKeys:[NSArray arrayWithObjects:@"attributes", @"geometry", nil]];
    
    return jsonDict;
}

-(NSString *) convertNSDictToEncodedJson:(NSDictionary *)dict {
    
    NSString *jsonString = nil;
    
    jsonString = [NSString stringWithFormat: @"[%@]" , [dict ags_JSONRepresentation]];
    
    // Per la conversió x-www-form-urlencoded utilitzem la categoria NSString + UrlEncoding
    jsonString = [NSString stringWithFormat: @"features=%@&gdbVersion=&rollbackOnFailure=true&f=pjson" , [jsonString stringByUrlEncoding]];
    
    return jsonString;
}

#pragma mark NSURLConnection Delegate Methods

// Mètdoes estàndards de la classe delegada de NSURLConnection
- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response
{
    // This method is called when the server has determined that it
    // has enough information to create the NSURLResponse.
	
    // It can be called multiple times, for example in the case of a
    // redirect, so each time we reset the data.
	
    // receivedData is an instance variable declared elsewhere.
    
    [receivedData setLength:0];
}

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data
{
    // Append the new data to receivedData.
    // receivedData is an instance variable declared elsewhere.
    
    [receivedData appendData:data];
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
    // release the connection, and the data object
    // receivedData is declared as a method instance elsewhere
    
    // Informem en el Log i deixem la captura sense atendre: setStatusFirstUnnattendedItem + processCaptures
    NSLog(@"Connection failed! Error - %@ %@",
          [error localizedDescription],
          [[error userInfo] objectForKey:NSURLErrorFailingURLStringErrorKey]);
}

// En cas cas d'error no actualitzem l'estatus de la captura i referesquem la llista
- (void)connectionDidFinishLoading:(NSURLConnection *)connection
{
    NSLog(@"Connection OK");

    long objectid = [self getObjectIdFromResponse:receivedData];

    if (objectid != 0) {
        MyManager *sharedManager = [MyManager sharedManager];
        
        // Setejar l'objectId obtingut a l'status de la captura i desapareixerà de la llista unnattendedList
        [sharedManager.myCaptureList setStatusFirstUnnattendedItem:(int)objectid];
        [[NSNotificationCenter defaultCenter] postNotificationName:@"dealNotification" object: nil];
        
        // Si tenim connexió i una vegada retirada la captura tractada de unnattendedList
        // tornem a tractar el primer element de unnattendedList si != nil
        [self processCaptures];
    }
}


// Obtenim l'ObjectId enviat com a resposta pel servidor ArcGIS i l'utilitzem per marcar l'status de la captura
-(long)getObjectIdFromResponse:(NSMutableData *) recData {
    
    bool success = false;
    long objectid = 0;
    
    NSString *messsage = [[NSString alloc] initWithFormat:@"Received %d bytes of data",[recData length]];

    NSString *responseString = [[NSString alloc] initWithData:recData encoding:NSASCIIStringEncoding];
    NSLog(@"Response: %@", responseString);
    
    // Seria millor buscar una altre forme de comprovar quan ens envien un JSON amb l'ObjectId i quan ens envien un error, 
    // però com les estructures són diferents, llegir com a diccionari donaria una excepció
    if ([recData length] != 72) {
        NSLog(@"Error! %@", messsage);
        return objectid;
    } else {
        NSLog(@"Succeeded! %@", messsage);
    }
    
    // Converteix JSON a Diccionari
    NSDictionary *jsonDict = [responseString ags_JSONValue];
    
    NSString *objectidString = @"objectId";
    NSString *successString = @"success";

//    NSMutableArray *keys = [[NSMutableArray alloc] init];
//    NSMutableArray *vals = [[NSMutableArray alloc] init];
//    
//    
//    NSDictionary* resultDict = [[jsonDict allValues] objectAtIndex:0];
//    
//    if ([recData length] != 0) {
//        keys = [[resultDict allKeys] objectAtIndex:0];
//        vals = [[resultDict allValues] objectAtIndex:0];
//        for (int ind = 1; ind <= keys.count; ind++) {
//            if ([[keys objectAtIndex:ind] isEqualToString:objectidString]) {
//                objectid = [[vals objectAtIndex:ind] longValue];
//            }
//        }
//    }
    
    NSDictionary* resultDict = [[jsonDict allValues] objectAtIndex:0];
    
    
    for (NSDictionary *element in resultDict) {
        
        if([element valueForKey:objectidString] != nil) {
            objectid = [[element valueForKey:objectidString] longValue];
            NSLog(@"objectid: %ld", objectid);
        }
        if([element valueForKey:successString] != nil) {
            success = [[element valueForKey:successString] boolValue];
            NSLog(@"success: %s", success ? "true" : "false");
        }
        
    }
    
    if (!success) {
        NSLog(@"Return value error. \"Result\" didn't found");
        return 0;
    }
    
    if (objectid == 0) {
        NSLog(@"Return value error. \"objectId\" didn't found");
    }
    
    return objectid;
}
@end
