//
//  TWSettingsData.m
//  TrailWalker
//
//  Created by Roger Riera Diaz on 10/05/13.
//  Copyright (c) 2013 Roger Riera Diaz. All rights reserved.
//

#import "TWSettingsData.h"

@implementation TWSettingsData

@synthesize accuracy;
@synthesize team;
@synthesize locService;
@synthesize frequency;
@synthesize filePath;
@synthesize uuid;


- (id)init
{
    self = [super init];
    if (self)
    {
        // Estableix Documents com a directori del plist
        filePath = [[self docsDir] stringByAppendingPathComponent:@"TrailWalker-Settings.plist"];
        
        // Inicialment utilitzavem la plist del Main Bundle però és només de lectura!
//        = [[NSBundle mainBundle] pathForResource:@"TrailWalker-Settings" ofType:@"plist"];
        
        // La primera execució de l'app, copia el plist del Main Bundle al directori Documents de l'aparell
        if (![[NSFileManager defaultManager] fileExistsAtPath:filePath]) {
            [[NSFileManager defaultManager] copyItemAtPath:[[NSBundle mainBundle] pathForResource:@"TrailWalker-Settings" ofType:@"plist"]  toPath:filePath error:nil];
        }
        
        self.dictionary = [NSMutableDictionary dictionaryWithContentsOfFile:filePath];
        
        // Generem el UUID i el registrem en el diccionari
        [self setUuid:[self uuidString]];
        [self.dictionary setObject:uuid forKey:@"UUID"];
        [self.dictionary writeToFile:filePath atomically: YES];

    }
    return self;
}

- (NSString *)uuidString {
    // Genera un UUID únic per l'app la primera vegada que s'executa.    
    id appIdObject = [NSUUID UUID];
    NSString *appIdString = [appIdObject UUIDString];
    
    return appIdString;
}

-(NSString *)docsDir {
    return [NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES) objectAtIndex:0];
}

// Obté totes les dades del plist de documents i seteja els atributs de la classe
-(void)getDataFromDict {

    NSNumber *accuracyInfo;
    NSString *teamInfo;
    NSString *languageInfo;
    NSNumber *frequencyInfo;
    NSString *uuidInfo;

    accuracyInfo = [self.dictionary objectForKey:@"Accuracy"];
    [self setAccuracy:accuracyInfo];
        
    teamInfo = [self.dictionary objectForKey:@"Team"];
    [self setTeam:teamInfo];
    
    languageInfo = [self.dictionary objectForKey:@"LocateService"];
    [self setLocService:languageInfo];
    
    frequencyInfo = [self.dictionary objectForKey:@"Frequency"];
    [self setFrequency:frequencyInfo];
    
    uuidInfo = [self.dictionary objectForKey:@"UUID"];
    [self setUuid:uuidInfo];
}

// A partir atributs de la classe, actualitza les dades del plist de documents
-(void)setDataOnDict {
    
    [self.dictionary setObject:team forKey:@"Team"];
    [self.dictionary setObject:accuracy forKey:@"Accuracy"];
    [self.dictionary setObject:locService forKey:@"LocateService"];
    [self.dictionary setObject:frequency forKey:@"Frequency"];
    [self.dictionary setObject:uuid forKey:@"UUID"];
    
    [self.dictionary writeToFile:filePath atomically: YES];

}

@end
