//
//  TWSettingsViewController.m
//  TrailWalker
//
//  Created by Roger Riera Diaz on 21/04/13.
//  Copyright (c) 2013 Roger Riera Diaz. All rights reserved.
//

#import "TWSettingsViewController.h"

@interface TWSettingsViewController ()

@property (weak, nonatomic) IBOutlet UILabel *accuracyLabel;
@property (weak, nonatomic) IBOutlet UILabel *frequencyLabel;
@property (weak, nonatomic) IBOutlet UILabel *serviceLabel;
@property (weak, nonatomic) IBOutlet UILabel *teamLabel;

@property (weak, nonatomic) IBOutlet UITableViewCell *frequencyCell;
@property (weak, nonatomic) IBOutlet UITableViewCell *accuracyCell;

@end

@implementation TWSettingsViewController


- (void)viewDidLoad
{
    [super viewDidLoad];
    
    // La pantalla de paràmetres carrega totes les dades a partir de la plist de la Singleton
    MyManager *sharedManager = [MyManager sharedManager];
    self.accuracyLabel.text = [NSString stringWithFormat: @"%@ m" , sharedManager.mySettingData.accuracy.stringValue];
    self.frequencyLabel.text = [NSString stringWithFormat: @"%@ min" , sharedManager.mySettingData.frequency.stringValue];
    self.serviceLabel.text = sharedManager.mySettingData.locService;
    self.teamLabel.text = sharedManager.mySettingData.team;
    
    // Si el sistema no és l'estàndard, la primera vegada, deshabilitem els paràmetres de la captura
    if (![sharedManager.mySettingData.locService isEqualToString:@"Standard"]) {
        self.frequencyCell.userInteractionEnabled = NO;
        self.frequencyCell.textLabel.enabled = NO;
        self.frequencyCell.detailTextLabel.enabled = NO;
        
        self.accuracyCell.userInteractionEnabled = NO;
        self.accuracyCell.textLabel.enabled = NO;
        self.accuracyCell.detailTextLabel.enabled = NO;
    }
    
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}


// En funció del segue detectat naveguem a una classe dels subsettings o un altre
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender
{
    MyManager *sharedManager = [MyManager sharedManager];
	if ([segue.identifier isEqualToString:@"teamSegue"])
	{
		UINavigationController *navigationController = segue.destinationViewController;
		TWTeamController *TWTeamController = [[navigationController viewControllers] objectAtIndex:0];
		TWTeamController.delegate = self;
	} else if ([segue.identifier isEqualToString:@"accuracySegue"]) {
		UINavigationController *navigationController = segue.destinationViewController;
		TWAccuracyController *TWAccuracyController = [[navigationController viewControllers] objectAtIndex:0];
		TWAccuracyController.delegate = self;
	} else if ([segue.identifier isEqualToString:@"locationSegue"]) {
        TWLocationServiceController *TWLocationController = segue.destinationViewController;
		TWLocationController.delegate = self;
		TWLocationController.locationService = sharedManager.mySettingData.locService;
	} else if ([segue.identifier isEqualToString:@"frequencySegue"]) {
        UINavigationController *navigationController = segue.destinationViewController;
		TWTeamController *TWTeamController = [[navigationController viewControllers] objectAtIndex:0];
		TWTeamController.delegate = self;
	}
}


#pragma mark - TWTeamControllerDelegate

// Metode per la classe delegada que enregistre el nom de l'equip
- (void)TWTeamController:
(TWTeamController *)controller
           didReturnTeam:(NSString *)teamName;
{
    MyManager *sharedManager = [MyManager sharedManager];
    [sharedManager.mySettingData setTeam:teamName];
    [sharedManager.mySettingData setDataOnDict];
    self.teamLabel.text = teamName;
    
	[self dismissViewControllerAnimated:YES completion:nil];
}

- (void)teamControllerDidCancel:
(TWTeamController *)controller
{
	[self dismissViewControllerAnimated:YES completion:nil];
}

#pragma mark - TWAccuracyControllerDelegate

- (void)accuracyControllerDidCancel:
(TWAccuracyController *)controller
{
	[self dismissViewControllerAnimated:YES completion:nil];
}

// Metode per la classe delegada que enregistre la precissió de l'equip
- (void)TWAccuracyController:
(TWAccuracyController *)controller
           didReturnAccuracy:(NSNumber *)accuracyVal;
{
    MyManager *sharedManager = [MyManager sharedManager];
    [sharedManager.mySettingData setAccuracy:accuracyVal];
    [sharedManager.mySettingData setDataOnDict];    
    self.accuracyLabel.text = [NSString stringWithFormat: @"%@ m" , accuracyVal.stringValue];
    
	[self dismissViewControllerAnimated:YES completion:nil];
}


#pragma mark - TWLocationControllerDelegate

// Metode per la classe delegada que enregistre el servei de localització
- (void)TWLocationController:
(TWLocationServiceController *)controller
                   didSelectService:(NSString *)theService
{
    MyManager *sharedManager = [MyManager sharedManager];
    [sharedManager.mySettingData setLocService:theService];
    [sharedManager.mySettingData setDataOnDict];
    self.serviceLabel.text = theService;
    
	[self.navigationController popViewControllerAnimated:YES];
    
    // Si el sistema no és l'estàndard, deshabilitem els paràmetres de la captura, si no, els activem
    if (![theService isEqualToString:@"Standard"]) {
        self.frequencyCell.userInteractionEnabled = NO;
        self.frequencyCell.textLabel.enabled = NO;
        self.frequencyCell.detailTextLabel.enabled = NO;
        
        self.accuracyCell.userInteractionEnabled = NO;
        self.accuracyCell.textLabel.enabled = NO;
        self.accuracyCell.detailTextLabel.enabled = NO;
    } else {
        self.frequencyCell.userInteractionEnabled = YES;
        self.frequencyCell.textLabel.enabled = YES;
        self.frequencyCell.detailTextLabel.enabled = YES;
        
        self.accuracyCell.userInteractionEnabled = YES;
        self.accuracyCell.textLabel.enabled = YES;
        self.accuracyCell.detailTextLabel.enabled = YES;
    }
}

// Metode per la classe delegada que enregistre la freqüència de captura
#pragma mark - TWFrequencyControllerDelegate

- (void)frequencyControllerDidCancel:
(TWFrequencyController *)controller
{
	[self dismissViewControllerAnimated:YES completion:nil];
}

- (void)TWFrequencyController:
(TWFrequencyController *)controller
           didReturnFrequency:(NSNumber *)frequencyVal;
{
    MyManager *sharedManager = [MyManager sharedManager];
    [sharedManager.mySettingData setFrequency:frequencyVal];
    [sharedManager.mySettingData setDataOnDict];    
    self.frequencyLabel.text = [NSString stringWithFormat: @"%@ min" , frequencyVal.stringValue];
    
	[self dismissViewControllerAnimated:YES completion:nil];
}


@end